#!/usr/bin/perl -w

#
# Amazon wishlist exporter - 12/6/05
#
# This script is designed to take as input a list of Amazon wishlist ids (from
# the command line and/or an input file), download the wishlist data and
# display it in either plaintext or a formatted HTML file. This is designed to
# replace the (current) lack of an export function on the Amazon site.
#
# Requirements:
#       * Needs both the CGI and Net::Amazon modules installed
#       * You need to insert your very own Amazon API access key,
#         obtained from http://www.amazon.com/soap (it's free)
#       * Only tested under Linux, probably works under Windows
#

use strict;
use Getopt::Std;
use Net::Amazon;
use Net::Amazon::Request::Wishlist;
use CGI;

# -----------------------------------------------------------------------------
# GLOBAL CONSTANTS
# -----------------------------------------------------------------------------
my $PROG_NAME = "wishlist.pl";
my $ACCESS_KEY = ""; # Obtained from www.amazon.com/soap

# -----------------------------------------------------------------------------
# GLOBAL VARIABLES
# -----------------------------------------------------------------------------
my %opts;
my @wishlists;
my $input_file;
my $display_text;
my $display_html;

my $num_items = 0;
my %wishlist_items;

# -----------------------------------------------------------------------------
# Main Program
# -----------------------------------------------------------------------------
&get_arguments();
&dump_wishlists();

&print_plaintext() if $display_text;
&print_html() if $display_html;

# -----------------------------------------------------------------------------
# Extract all wishlist information to hash using Amazon API
# -----------------------------------------------------------------------------
sub dump_wishlists {
        my $wishlist;
        my ($c, $q, $r); # Connection, query, response
        my $item;

        $c = Net::Amazon->new(token => $ACCESS_KEY);

        foreach $wishlist (@wishlists) {
                $q = Net::Amazon::Request::Wishlist->new(id => $wishlist);

                # Response is of type Net::Amazon::ASIN::Response
                my $r = $c->request($q);

                if($r->is_success()) {
                        foreach $item ($r->properties) {
                                $num_items++;

                                push(@{$wishlist_items{$item->Catalog()}}, $item);
                        }
                } else {
                        print "Error: ", $r->message(), "\n";
                }

                sleep(2); # Be nice to Amazon
        }
}

# -----------------------------------------------------------------------------
# Print wishlist data to a plaintext output file
# -----------------------------------------------------------------------------
sub print_plaintext {
        my $catalog;
        my $item;

        print "Total Items: $num_items\n";
        print '-' x 80, "\n\n\n";
        foreach $catalog (sort keys %wishlist_items) {
                print "$catalog (", scalar @{$wishlist_items{$catalog}}, ")\n";

                foreach $item ( @{$wishlist_items{$catalog}} ) {
                        print '-' x 80, "\n";
                        print $item->ProductName();
                        if (exists $item->{year}) {
                                print " [", $item->year(), "]" unless ($item->year() eq "");
                        }
                        print "\n";

                        if ($catalog eq "Book") {
                                print "\tAuthor:\t", join("/", $item->authors()), "\n";
                                print "\tPublisher:\t", $item->Manufacturer(), "\n";
                        } elsif ($catalog eq "Software") {
                                print "\tPublisher:\t", $item->Manufacturer(), "\n";
                        } elsif ($catalog eq "DVD") {
                                print "\tPublisher:\t", $item->Manufacturer(), "\n";
                        } elsif ($catalog eq "Music") {
                                print "\tMusic Label:\t", $item->Manufacturer(), "\n";
                        } else {
                                print "\tManufacturer:\t", $item->Manufacturer(), "\n";
                        }

                        if (exists $item->{OurPrice}) {
                                print "\tPrice:\t", $item->OurPrice(), "\n";
                                if (exists $item->{UsedPrice}) {
                                        print "Used Price: ", $item->UsedPrice(), "\n";
                                }
                        }

                        print "\tASIN:\t", $item->Asin(), "\n";
                }
                print "\n\n";
        }
        print '-' x 80, "\n";
        print "Total Items: $num_items\n";
        print '-' x 80, "\n";
}

# -----------------------------------------------------------------------------
# Print wishlist data to a HTML formatted output file
# -----------------------------------------------------------------------------
sub print_html {
        my $q = new CGI;
        my $catalog;
        my $item;

        print $q->start_html(-title=>'Amazon Wishlist',
                             -style=>{'src'=>'wishlist.css'}),
              $q->h3({-id=>'header'}, "Total Items: $num_items");

        foreach $catalog (sort keys %wishlist_items) {
                print $q->start_div({-class=>'catalog'}),
                      $q->h2("$catalog (", scalar @{$wishlist_items{$catalog}}, ")"),
                      $q->start_ol;

                foreach $item ( @{$wishlist_items{$catalog}} ) {
                        print $q->start_li({-class=>'item'}),
                              $q->start_a({-href=>$item->url()}),
                              $q->img({-src=>$item->ImageUrlSmall(),
                                       -alt=>"Thumbnail of " . $item->ProductName()}),
                              $q->span({-class=>'item_name'}, $item->ProductName());

                        if (exists $item->{year}) {
                                print $q->span({-class=>'item_year'},
                                               $item->year()) unless ($item->year() eq "");
                        }

                        if ($catalog eq "Book") {
                                print $q->span({-class=>'item_author'},
                                               "Author: ".join("/", $item->authors()));
                                print $q->span({-class=>'item_manufacturer'},
                                               "Publisher: ".$item->Manufacturer());
                        } elsif ($catalog eq "Software") {
                                print $q->span({-class=>'item_manufacturer'},
                                               "Publisher: ".$item->Manufacturer());
                        } elsif ($catalog eq "DVD") {
                                print $q->span({-class=>'item_manufacturer'},
                                               "Publisher: ".$item->Manufacturer());
                        } elsif ($catalog eq "Music") {
                                print $q->span({-class=>'item_manufacturer'},
                                               "Music Label: ".$item->Manufacturer());
                        } else {
                                print $q->span({-class=>'item_manufacturer'},
                                               "Manufacturer: ".$item->Manufacturer());
                        }

                        if (exists $item->{OurPrice}) {
                                print $q->span({-class=>'item_price'},
                                               "Price: ".$item->OurPrice());
                                if (exists $item->{UsedPrice}) {
                                        print $q->span({-class=>'item_price_used'},
                                                       "Used Price: ".$item->UsedPrice());
                                }
                        }
                        print $q->span({-class=>'item_asin'}, "ASIN: ".$item->Asin()),
                              $q->end_a,
                              $q->end_li;
                }
                print $q->end_ol,
                      $q->end_div;
        }

        print $q->h3({-id=>'footer'}, "Total Items: $num_items"),
              $q->end_html;
}

# -----------------------------------------------------------------------------
# Retrieve and process command line arguments
# -----------------------------------------------------------------------------
sub get_arguments {
        getopts('fhi:p', \%opts) or &print_usage();

        # Print help/usage information to the screen if necessary
        &print_usage() if ($opts{h});
        &print_usage() unless ($ARGV[0] or $opts{i});

        # Copy command line arguments to internal variables
        $input_file = 0 unless ($input_file = $opts{i});
        $display_text = 0 unless ($display_text = $opts{p});
        $display_html = 0 unless ($display_html = $opts{f});

        # Read input file and merge with command line as necessary
        if ($input_file) {
                open(INFILE, "$input_file") || die "\nError: Cannot open $input_file - $!\n";
                        @wishlists = <INFILE>;
                close(INFILE);

                if ($ARGV[0]) {
                        push(@wishlists, @ARGV);
                }
        } else {
                @wishlists = @ARGV;
        }

        # If error with output types, default to plaintext output
        if (!$display_text && !$display_html) {
                $display_text = 1;
                $display_html = 0;
        }
        if ($display_text && $display_html) {
                $display_text = 1;
                $display_html = 0;
        }
}

# -----------------------------------------------------------------------------
# Print usage/help information to the screen and exit
# -----------------------------------------------------------------------------
sub print_usage {
        die <<USAGE;
Usage: $PROG_NAME [-fhp] [-i file] [wishlist_ids]
  -f ... output HTML formatted wishlist
  -h ... display this help information
  -i ... input file of newline delimited wishlist identifiers
  -p ... output plaintext formatted wishlist
USAGE
}
